<?php
/*
 +-------------------------------------------------------------------------+
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/
/*******************************************************************************
    Author ......... Jean-Michel Pepin classes plugin for network devices (PEPJ in forum.cacti.net & http://cactiusers.org/forums/)
    Modified ....... .
    Program ........ class definition for Cisco network devices in PHP for Cacti or PHP software

    congratulation: For the TFTP part theses classes are based on the work of Pancho http://www.pancho.org
    & cacti http://www.cacti.net, 
    Thanks to all the developpers of cacti and pancho who make my work much easier for TFTP. pancho TFTP cisco catalyst Kevin Thayer
reference http://www.cisco.com/en/US/tech/tk648/tk362/technologies_tech_note09186a0080094aa6.shtml
reference http://www.cisco.com/en/US/tech/tk648/tk362/technologies_tech_note09186a0080094aa3.shtml
reference http://www.oidview.com/mibs/9/CISCO-VLAN-MEMBERSHIP-MIB.html
reference http://tools.cisco.com/Support/SNMP/do/BrowseMIB.do?local=en&step=2&mibName=CISCO-ENVMON-MIB
*******************************************************************************/

/* define the OID of the vendor */
$toidvendor['Cisco']        = '.1.3.6.1.4.1.9';
//$toidvendor['arrowpoint']   = '.1.3.6.1.4.1.9.9.368'; // CSS Cisco
$toidtovendor['.1.3.6.1.4.1.9'] = 'Cisco';

class cisconetworkclass
{
   function vendorname($FunctionName)
   {
        return "Cisco";       
   }
   function showclass()
   {
        return "this is a Cisco class";       
   }
   function priv($obj,$FunctionName,$param="")
   {
   	$fct = "Cisco".$FunctionName;
       	if (function_exists($fct)) $ret = ($param=="") ? $fct($obj) : $fct($obj,$param);
       	else {
       	  $ret=0;
          $obj->result= array();
          $obj->errormsg = "Error: Function does not exist.";
          $obj->retcode = $ret;
       	}
        return $ret;
   }
   function tftp($obj)
   {
     global $networkclass_isdebug,$networkclass_islog;
     if ($obj->inittftp) {
      $typtransfert = CiscoChecktypdevice($obj);
      if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG tftp typtransfert=".$typtransfert."\n");
      switch ($typtransfert) {
	case "ciscoIOSdeprecated":
	  CiscoTFTPtransfer_deprecated($obj);
	  break;
	case "ciscoIOS":
	  CiscoTFTPIOStransfert($obj);
	  break;
	case "ciscoCat":
	  CiscoTFTPCatOStransfert($obj);
	  break;
	default:
	   $obj->result= "unknow device";
	   $obj->retcode = 0;
	  break;
      }
     }
   }
   /* return if the device is a host */
   function DeviceIsHost($objdevice,$init=false) {
     global $networkclass_isdebug,$networkclass_islog;
     // NEVER for CISCO class (only network devices)

     // the two following value could help you to find it
     // $obj->description();  // to be sure description has been ask at least one time
     // $description = $obj->result; or $description = $obj->description;
     // $oiddevice = $objdevice->oiddevice;
     //force debug $objdevice->setdebug(true,true);
     // if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG DeviceIsHost ->".print_r($valmib,true)."\n");
     if ($init) { $objdevice->ishost = false; }
     $result= $objdevice->ishost;
     return $result;
   }
}

/* return the type of the device (used only for the function in this class in case of a specific behaviour)
*/
function CiscoChecktypdevice($obj) {
   $obj->description();  // to be sure description has been ask at least one time
   $description = $obj->result;
   $typdevice = "unknow";
   $patternold= '/Version 1(?:1|0)/';
   $pattern1= '/C(?:35|29)50.*Version 12.[1-9]\(1[2-9]/'; //Catalyst 2950/3550 with 12.1(12) or higher
   $pattern2= '/ C(?:(?:800)|35(?:(?:50)|(?:00XL))) /'; //Catalyst 2900XL/3500XL
   $pattern3= '/Version 1\d/';
   $patternCat= '/(?:Catalyst)|(?:WS-)/i'; // rest of catalysts
   switch (true) {
	case preg_match($patternold,$description)>0:
	  $typdevice = "ciscoIOSdeprecated";
	  break;
	case preg_match($pattern1,$description)>0:
	  $typdevice = "ciscoIOS";
	  break;
	case preg_match($pattern2,$description)>0:
	  $typdevice = "ciscoIOSdeprecated";
	  break;
	case preg_match($pattern3,$description)>0:
	  $typdevice = "ciscoIOS";
	  break;
	case preg_match($patternCat,$description)>0:
	  $typdevice = "ciscoCat";
	  break;
	default:
	  break;
   }
   return $typdevice;
}

function CiscoisCat29xx($obj) {
   $obj->description();  // to be sure description has been ask at least one time
   $description = $obj->result;
   $typdevice = "no";
   $pattern0= '/(WS-C2948)/i'; //Catalyst 2948
   $pattern1= '/C(29\d\d)/i'; //Catalyst 29xx
   $pattern2= '/C(?:|35(?:(?:50)|(?:00XL)))/i'; //Catalyst 2900XL/3500XL
   switch (true) {
	case preg_match($pattern0,$description)>0:
	  $typdevice = "2948";
	  break;
	case preg_match($pattern1,$description)>0:
	  $typdevice = "29xx";
	  break;
	case preg_match($pattern2,$description)>0:
	  $typdevice = "2900XL-3500XL";
	  break;
	default:
	  break;
   }
   return $typdevice;
}

/* ************************************************************************
 Duplex : should return an array like
 $result[ifindex] = 0 for unknow, 1 for half duplex, 2 for full duplex, 3 auto;
*/
function CiscoDuplex($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//$objdevice->setdebug(true,true);  //force manual debug
  $snmpok=0;
  $result = array();
  $msg = "Error: Duplex no response";
  $Ciscotype= CiscoisCat29xx($objdevice);
  if ($Ciscotype=="29xx") {
    $a_xmlmib = array(array("source"=>"CISCO-C2900-MIB.xml"));
    $mib="c2900PortDuplexStatus";
    $mib2="c2900PortSwitchPortIndex";
  } else {
   $a_xmlmib = array(array("source"=>"ciscoStackMIB.xml"));
   $mib="portDuplex";
   $mib2="portIfIndex";
  }
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Duplex Load MIB ->".print_r($a_xmlmib,true)."\n");
  searchin_xmlMIB($a_xmlmib,$mib,$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Duplex MIB search of $mib ->".print_r($valmib,true)."\n");
  searchin_xmlMIB($a_xmlmib,$mib2,$valmib2);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Duplex index MIB search of $mib2 ->".print_r($valmib2,true)."\n");
  $snmpok=(sizeof($valmib)>0) && (sizeof($valmib2)>0);
  if ($snmpok) {
   networkclass_give_snmp($objdevice,$valmib);
   $pDuplex = $objdevice->result;
   $snmpok1 = $objdevice->retcode;
   networkclass_give_snmp($objdevice,$valmib2);
   $pDuplexindex = $objdevice->result;
   $snmpok2 = $objdevice->retcode;
   $snmpok = $snmpok1 && $snmpok2;
   if ($snmpok) {
     if ((sizeof($pDuplex)>0) && (sizeof($pDuplexindex)>0)) {
        remove_oid_for_find_index($valmib["oid"],$pDuplex,$result1);
        remove_oid_for_find_index($valmib2["oid"],$pDuplexindex,$result2);
        foreach ($result2 as $index => $val) {
        	$result[$val] = $result1[$index];
        }
     }
     if ($Ciscotype=="29xx") {
       // conversion for independancy reason
       foreach ($result as $index => $val) {
         switch ($val) {
          case 1: // full duplex
	    $res= 2;
	    break;
	  case 2: // half duplex
	    $res= 1;
	    break;
	  default:    //  0       other or unknown
	     $res= 0;
	    break;
         }  
        $result[$index] =$res;
       }
     } else {
       // conversion for independancy reason
       foreach ($result as $index => $val) {
         switch ($val) {
	  case 1: // half duplex
	    $res= 1;
	    break;
	  case 2: // full duplex
	    $res= 2;
	    break;
	  case 4: // auto duplex
	    $res= 3;
	    break;
	  default:    //  0       other or unknown
	     $res= 0;
	    break;   
        }
        $result[$index] =$res;
       }
    }
    $msg = "";
   }
  }

 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Duplex result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 Trunk : should return an array like in order to be independant for vendors
 $result[interfaceindex] = "0" no trunk "1","2",... for other state;
*/
function CiscoTrunk($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//$objdevice->setdebug(true,true); // force debug 
 $mib="vlanTrunkPortDynamicStatus";
 $a_xmlmib = array(array("source"=>"CISCO-VTP-MIB.xml"));
 $snmpok=0;
 $msg = "";
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG CiscoTrunk Load MIB ->".print_r($a_xmlmib,true)."\n");
 $mib = "vlanTrunkPortDynamicStatus";
 searchin_xmlMIB($a_xmlmib,$mib,$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG CiscoTrunk MIB search of $mib ->".print_r($valmib,true).", result=>".$objdevice->result."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   /* Cisco give 1 : trunking 2 : notTrunking */
   /* we want "0" no trunk "1","2",... for other state" */
   convert_index_to_ifindex($objdevice->result,$result);
   foreach ($result as $index => $value) if ($result[$index]=="2") $result[$index]="0";
   $snmpok=$objdevice->retcode;
   $msg = $objdevice->errormsg;
 } else {
    $result= array();
    $msg = "Error: MIB not exist.";
 }
 $objdevice->result= $result;
 if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG CiscoTrunk result ->".print_r($result,true)."\n");
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 VLAN : should return an array like in order to be independant for vendors
 $result["state"][vlanindex] = "0" for no VLAN or VLAN number;
 $result["vlanname"][vlanindex] = "0" for no VLAN or VLAN number;
 $result["index"][vlanindex] = ifindex;
 $result["nbport_by_vlan"][vlanindex] = nb ports in vlan
 $result["port_by_vlan"][vlanindex][] = array of ports in vlan
 $result["vlan_by_port"][vlanindex][] = array of ports in vlan
 $result["voiceVLAN"][] = array of Interface witch voice VLAN (Catalyst 29xx)
 */
function CiscoVLAN($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//$objdevice->setdebug(true,true); // for manually debug
 $vlanstate= array(); 
 $vlanname = array(); 
 $vlanindex = array();
 $arrayofnbportsinvlan = array();
 $arrayofportsinvlan = array();
 $snmpok=0;
 $msg = "Error: MIB not exist.";
 $a_xmlmib = array(array("source"=>"CISCO-VTP-MIB.xml"),array("source"=>"CISCO-C2900-MIB.xml"));
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN Load MIB ->".print_r($a_xmlmib,true)."\n");
 /* search the VLAN information */
 searchin_xmlMIB($a_xmlmib,"vtpVlanState",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN state MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok1=$objdevice->retcode;
   /* Cisco give operational 1 suspended 2 mtuTooBigForDevice 3 mtuTooBigForTrunk 4 */
   /* we want "0" no vlan active "1","2",... for other state" */
   if ($snmpok1) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$vlanstate);
        foreach ($vlanstate as $index => $value) if ($vlanstate[$index]!="1") $vlanstate[$index]="0";
        $msg= "";
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN state array ->".print_r($vlanstate,true)."\n");
 }
 searchin_xmlMIB($a_xmlmib,"vtpVlanName",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN name MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok2=$objdevice->retcode;
   if ($snmpok2) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$vlanname);
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN vlanname ->".print_r($vlanname,true)."\n");
 }
 searchin_xmlMIB($a_xmlmib,"vtpVlanIfIndex",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN vtpVlanIfIndex MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok3=$objdevice->retcode;
   if ($snmpok3) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$vlanindex);
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN vtpVlanIfIndex ->".print_r($vlanindex,true)."\n");
 }
 /* VLAN based */
 $arrayofnbportsinvlan= array();
 $arrayofportsinvlan= array();
 if (sizeof($vlanindex)>0) {
   networkclass_searchinMIB("dot1dBaseNumPorts",$valmib);
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN dot1dBaseNumPorts MIB is ->".print_r($valmib,true)."\n");
   $snmpok4 = networkclass_give_snmp($objdevice,$valmib);
   if (sizeof($valmib)>0) {
     if ($objdevice->param["version"]!="3") {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity();
     } else  {
      $newcommunity = $objdevice->snmp->getsnmp3context();
     }
    foreach ($vlanindex as $vlan => $ifIndex) {
     if ($objdevice->param["version"]!="3") {
      $newcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity."@".$vlan);
     } else  {
      $newcommunity = $objdevice->snmp->setsnmp3context("vlan-".$vlan);
     }
     if (intval($vlan)<1000) {
     	networkclass_give_snmp($objdevice,$valmib);
        if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN dot1dBaseNumPorts value of Vlan$vlan is ->".print_r($objdevice->result,true)."\n");
        $arrayofnbportsinvlan[$vlan] = $objdevice->result;
     } else
        $arrayofnbportsinvlan[$vlan] = 0;
     //convert_index_to_ifindex($objdevice->result,$arrayofnbportsinvlan[$vlan]);
     $snmpok4=$objdevice->retcode;
    }
    if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco ($oldcommunity) VLAN dot1dBaseNumPorts for vlans ->".print_r($arrayofnbportsinvlan,true)."\n");	
     if ($objdevice->param["version"]!="3") {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity);
     } else  {
      $oldcommunity = $objdevice->snmp->setsnmp3context($oldcommunity);
     }
   }
   networkclass_searchinMIB("dot1dBasePortIfIndex",$valmib);
   $snmpok5 = networkclass_give_snmp($objdevice,$valmib);
   if (sizeof($valmib)>0) {
     if ($objdevice->param["version"]!="3") {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity();
     } else  {
      $oldcommunity = $objdevice->snmp->getsnmp3context();
     }
    foreach ($vlanindex as $vlan => $ifIndex) {
     if ($objdevice->param["version"]!="3") {
      $newcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity."@".$vlan);
     } else  {
      $newcommunity = $objdevice->snmp->setsnmp3context("vlan-".$vlan);
     }
     if (intval($vlan)<1000) {
        networkclass_give_snmp($objdevice,$valmib);
        //$arrayofportsinvlan[$vlan] = $objdevice->result;
        if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$arrayofportsinvlan[$vlan]);
     } else $arrayofportsinvlan[$vlan] = array();
     $snmpok5=$objdevice->retcode;
    }
    if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN dot1dBasePortIfIndex for vlans->".print_r($arrayofportsinvlan,true)."\n");	
     if ($objdevice->param["version"]!="3") {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity);
     } else  {
      $oldcommunity = $objdevice->snmp->setsnmp3context($oldcommunity);
     }
   }
 }
 if (CiscoisCat29xx($objdevice)=="29xx") {
   networkclass_searchinMIB("c2900PortVoiceVlanId",$valmib);
   $snmpok6 = networkclass_give_snmp($objdevice,$valmib);
   if (sizeof($valmib)>0) {
     if ($objdevice->param["version"]!="3") {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity();
     } else  {
      $oldcommunity = $objdevice->snmp->getsnmp3context();
     }
     networkclass_give_snmp($objdevice,$valmib);
     if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$voiceVLAN);
     //$result["voiceVLAN"] = $objdevice->result;
     $snmpok6=$objdevice->retcode;
     if ($snmpok6 && (($networkclass_isdebug)|| ($networkclass_islog)))
       networkclass_debug("DEBUG Cisco VLAN c2900PortVoiceVlanId for vlans->".print_r($voiceVLAN,true)."\n");	
  }
 }
 
 /* convert to ifindex !!!!!*/
 $result["index"] = @$vlanindex;
 $result["state"] = @$vlanstate;
 $result["VlanID"] = @$vlanname;
 $result["nbport_by_vlan"] = @$arrayofnbportsinvlan;
 $result["port_by_vlan"] = @$arrayofportsinvlan;
 $result["vlan_by_port"] = array();
 foreach ($arrayofportsinvlan as $vlanID => $tport) {
  foreach ($tport as $ndx => $port) {
   $result["vlan_by_port"][$port][] = $vlanID;
  }
 }
 if (isset($voiceVLAN)) $result["voiceVLAN"] = @voiceVLAN;
 $objdevice->result= $result;
 if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG Cisco VLAN result ->".print_r($result,true)."\n");
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok1;
 return $snmpok1;
}

/* ************************************************************************
 Channel : should return an array with all type of channel like ... in order
  to be independant for vendors who have severals channel types
 $result["channelindex"][0..x]=channeltype (at least "LACP")
 $result["LACP"]["AdminKey"][ifIndex] = AdminKey of LaCP channel;  // "dot3adAggPortActorAdminKey"
 $result["LACP"]["OperKey"][ifIndex] = OperKey of LaCP channel;    // "dot3adAggPortActorOperKey"
 $result["LACP"]["PortActorAdminState"][ifIndex] = Port Selected in which index of the LaCP channel // "dot3adAggPortActorAdminState"
 $result["LACP"]["PortActorOperState"][ifIndex] = Port Attached in which index of the LaCP channel // "dot3adAggPortActorOperState"
 
 $result["PAGP"]["pagpOperationMode"][ifIndex] = PaGP channel off (1) manual(2) or pagpOn(3);     "pagpEthcOperationMode"
 $result["PAGP"]["pagpGroupIfIndex"][ifIndex] = Port Attached in which index of the PaGP channel or 0 for not attached "pagpGroupIfIndex"
 ....
 */
function CiscoChannel($objdevice) {
 networkclass_LACP($objdevice);
 $snmpok1 = $objdevice->retcode;
 $lacp = $objdevice->result;
 $result["channelindex"][0] = "LACP";
 $result["LACP"] = $lacp;
 CiscoPaGP($objdevice);
 $snmpok2 = $objdevice->retcode;
 $pagp = $objdevice->result;
 if ($snmpok2) {
   $result["channelindex"][1] = "PAGP";
   $result["PAGP"] = $pagp;
 }
 $snmpok = $snmpok1 || $snmpok2;
 $objdevice->result =$result;
 if ($snmpok) $objdevice->errormsg = "";
 return $snmpok;
}

/* ************************************************************************
 LACP : should return an array like in order to be independant for vendors
 per default IEEE8023-LAG-MIB
 $result["AdminKey"][ifIndex] = AdminKey of LaCP channel;   "dot3adAggPortActorAdminKey"
 $result["OperKey"][ifIndex] = OperKey of LaCP channel;     "dot3adAggPortActorOperKey"
 $result["PortActorAdminState"][ifIndex] = Port Selected in which index of the LaCP channel
 $result["PortActorOperState"][ifIndex] = Port Attached in which index of the LaCP channel
 */
function CiscoLACP($objdevice) {
 networkclass_LACP($objdevice);
 return $objdevice->retcode;
}

/* ************************************************************************
 PaGP : should return an array like in order to be independant for vendors
 ciscoPagpMIB
 $result["pagpOperationMode"][ifIndex] = PaGP channel off (1) manual(2) or pagpOn(3);     "pagpEthcOperationMode"
 $result["PortActorOperState"][ifIndex] = Port Attached in which index of the PaGP channel or 0 for not attached "pagpGroupIfIndex"
 */
function CiscoPaGP($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//$objdevice->setdebug(true,true); // for manually debug
 $pagpOperationMode= array(); 
 $pagpGroupIfIndex = array();
 $snmpok=0;
 $msg = "Error: MIB not exist.";
 $a_xmlmib = array(array("source"=>"CISCO-PAGP-MIB.xml"));
// if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG PaGP Load MIB ->".print_r($a_xmlmib,true)."\n");
 /* search the PaGP information */
 searchin_xmlMIB($a_xmlmib,"pagpEthcOperationMode",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG PaGP pagpEthcOperationMode MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok=$objdevice->retcode;
   if ($snmpok) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$pagpOperationMode);
        $msg= "";
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG PaGP pagpEthcOperationMode array ->".print_r($pagpOperationMode,true)."\n");
 }
 if ($snmpok) {
  //  search the other data
  searchin_xmlMIB($a_xmlmib,"pagpGroupIfIndex",$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG PaGP pagpGroupIfIndex MIB is ->".print_r($valmib,true)."\n");
  if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok2=$objdevice->retcode;
   if ($snmpok2) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$pagpGroupIfIndex);
   	foreach ($pagpGroupIfIndex as $index => $val) {
   		if ($index==$val) $pagpGroupIfIndex[$index] = "0";
   	}
   }
   else $msg = $objdevice->errormsg;
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG PaGP pagpGroupIfIndex ->".print_r($pagpGroupIfIndex,true)."\n");
  }
 }
 /* convert to ifindex !!!!!*/
 $result["pagpOperationMode"] = @$pagpOperationMode;
 $result["PortActorOperState"] = @$pagpGroupIfIndex;
 $objdevice->result= $result;
// if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG PaGP result ->".print_r($result,true)."\n");
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 CPU : should return an array like
 $result["firstaveragename"] = value 1. average;
 $result["2averagename"] = value 2. average;
 $result["3averagename"] = value 3. average;
 .....
*/
function CiscoCPU($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $mib="cpmCPUTotalTable";
 $a_xmlmib = array(array("source"=>"CISCO-PROCESS-MIB.xml"));
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG CiscoCPU Load MIB ->".print_r($a_xmlmib,true)."\n");
 $snmpok=0;
 $result["5sec"] = 0;
 $result["1min"] = 0;
 $result["5min"] = 0;
 $msg = "Error: CPU no response";
 searchin_xmlMIB($a_xmlmib,$mib,$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG CiscoCPU MIB search of $mib ->".print_r($valmib,true)."\n");
 $snmpok=(sizeof($valmib)>0);
 if ($snmpok) {
   networkclass_give_snmp($objdevice,$valmib);
   $tresult = $objdevice->result;
   $snmpok = $objdevice->retcode;
   if ($snmpok) {
     $result["5sec"] = $tresult[1]["value"];
     $result["1min"] = $tresult[2]["value"];
     $result["5min"] = $tresult[3]["value"];
     $msg = "";
   }
 }
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG CPU result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 Temperature : should return an array of temperature (in degres Celsius) like
 $result["modulename"][0] = modulename; or ""
 $result["mean"][0] = 0;
 $result["min"][0] = value; or ""
 $result["max"][0] = value; or ""
 $result["state"][0] = value; or ""
PS: Fahrenheit to celsius (F - 32) / 1.8
*/
function CiscoTemperature($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $a_xmlmib = array(array("source"=>"CISCO-ENVMON-MIB.xml"));
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Temperature Load MIB ->".print_r($a_xmlmib,true)."\n");
 $result=array();
 $snmpok=0;
 $msg = "cisco EnvMonTemperatureStatusTable not enabled or not supported.";
 $mib = "ciscoEnvMonTemperatureStatusDescr";
 searchin_xmlMIB($a_xmlmib,$mib,$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Temperature MIB search of $mib ->".print_r($valmib,true)."\n");
 $snmpok1=(sizeof($valmib)>0);
 networkclass_give_snmp($objdevice,$valmib);
 $tresult1 = $objdevice->result;
 $ok1=$objdevice->retcode;

 if ($ok1) {
   $mib="ciscoEnvMonTemperatureStatusValue";
   searchin_xmlMIB($a_xmlmib,$mib,$valmib);
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Temperature MIB search of $mib ->".print_r($valmib,true)."\n");
   networkclass_give_snmp($objdevice,$valmib);
   $tresult2 = $objdevice->result;
   $ok2=$objdevice->retcode;
   if ($ok2) {
	  $snmpok=1;
	  $min=100;
	  $max=-100;
	  foreach($tresult1 as $index => $val)
	    $result["modulename"][$index] = $val["value"];
	  $mean=0;
	  $i=1;
	  foreach($tresult2 as $index => $val) {
	    $temp= $val["value"];
	    $result["value"][$index] = $temp;
	    $min= ($temp < $min) ? $temp:$min;
	    $max= ($temp > $max) ? $temp:$max;
	    if ($temp>0) { $i++; $mean+= $temp;}
	  }
	  $mean=$mean/$i;
	  $result["mean"][0] = $mean;
	  $result["min"][0] = $min;
	  $result["max"][0] = $max;
	  $msg = "";
   }
 }
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Temperature result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 Fan : should return an array of Fan state () like
 $result["description"][index] = decription where index is the index number of the fan
 $result["state"][index] = "ok" or "down" "not present" are mandatory for functioning / not functioning , or an another text
*/
function CiscoFan($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $a_xmlmib = array(array("source"=>"CISCO-ENVMON-MIB.xml"));
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Fan Load MIB ->".print_r($a_xmlmib,true)."\n");
 $snmpok=0;
 $msg = "cisco ciscoEnvMonFanStatusDescr not enabled or not supported.";
 $mib = "ciscoEnvMonFanStatusDescr";
 searchin_xmlMIB($a_xmlmib,$mib,$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Fan description MIB search of $mib ->".print_r($valmib,true)."\n");
 $snmpok1=(sizeof($valmib)>0);
 networkclass_give_snmp($objdevice,$valmib);
 if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$tdescription);
 else  $tdescription= array();
 $ok1=$objdevice->retcode;
 $tstate = array();
 if ($ok1) {
   $mib="ciscoEnvMonFanState";
   searchin_xmlMIB($a_xmlmib,$mib,$valmib);
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Fan state MIB search of $mib ->".print_r($valmib,true)."\n");
   networkclass_give_snmp($objdevice,$valmib);
   if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$tstate);
   $ok2=$objdevice->retcode;
   if ($ok2) {
     foreach($tstate as $index => $val) {
      switch ($val) {
        // by Cisco normal(1), warning(2), critical(3), shutdown(4), notPresent(5), notFunctioning(6)
      	case 1:
      	  $tstate[$index] = "ok";
      	  break;
      	case 2:
      	  $tstate[$index] = "warning";
      	  break;
      	case 3:
      	  $tstate[$index] = "critical";
      	  break;
      	case 4:
      	  $tstate[$index] = "shutdown";
      	  break;
      	case 5:
      	  $tstate[$index] = "not present";
      	  break;
      	default:
      	  $tstate[$index] = "not Functioning";
      }
     }
   }
 }
 $snmpok = $ok1 && $ok2;
 $result["description"] = $tdescription;
 $result["state"] = $tstate;
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Fan result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 RAM : should return an array of RAM (processor)
 $result["total"] = 
 $result["used"] = 
 $result["free"] = 
*/
function CiscoRAM($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
  $result = array();
  $snmpok=0;
  $msg = "Error: RAM status table not enabled or not supported.";
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco RAM"."\n");
   $a_xmlmib = array(array("source"=>"CISCO-MEMORY-POOL-MIB.xml"));
   searchin_xmlMIB($a_xmlmib,"ciscoMemoryPoolUsed",$valmib);
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco ciscoMemoryPoolUsed MIB is ->".print_r($valmib,true)."\n");
   if (sizeof($valmib)>0) {
     networkclass_give_snmp($objdevice,$valmib);
     if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$tused);
     $snmpok=$objdevice->retcode;
     if ($snmpok) {
     	$firstval= 0;
     	foreach ($tused as $indx => $val) $firstval=$val;
   	$result["used"] = $firstval; //processor value
     }
     else {
        $msg = $objdevice->errormsg;
     }
     searchin_xmlMIB($a_xmlmib,"ciscoMemoryPoolFree",$valmib);
     if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco ciscoMemoryPoolFree MIB is ->".print_r($valmib,true)."\n");
     if (sizeof($valmib)>0) {
       networkclass_give_snmp($objdevice,$valmib);
       if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$tfree);
       $snmpok=$objdevice->retcode;
       if ($snmpok) {
     	$firstval= 0;
     	foreach ($tfree as $indx => $val) $firstval+=$val;
     	$result["free"] = $firstval;
   	$result["total"] = $firstval+$result["used"]; //processor value
       }
       else {
        $msg = $objdevice->errormsg;
       }
       if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG Cisco RAM ->".print_r($result,true)."\n");
     }
   }
  $objdevice->result= $result;
  $objdevice->errormsg = $msg;
  $objdevice->retcode = $snmpok;
  return $snmpok;
}

/* ************************************************************************
 MAC : for the VlanID should return an array of MAC addresses like
 $result[interfaceindex][indexMAC] -> Macaddress where interfaceindex is the inteface index
*/
function CiscoMAC($objdevice,$vlanID) {
 networkclass_MAC($objdevice,$vlanID);
 return $objdevice->retcode;
}

/* ************************************************************************
 STPchanges : for the VlanID should return an array of 
 $result["TopChanges"] -> The total number of topology changes detected by this bridge since the management entity was last reset or initialized.
 $result["TimeSinceTopologyChange"] -> The time (in hundredths of a second) since the last time a topology change was detected by the bridge entity.
*/
function CiscoSTPchanges($objdevice,$vlanID="") {
 networkclass_STPchanges($objdevice,$vlanID);
 return $objdevice->retcode;
}

/* ************************************************************************
 Serialnumber : should return an array like
 $result[0..xx] = array("description" -> description, "SN" -> serial number);
 IN DEVELOPEMENT
*/
/*
function CiscoSerialnumber($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//$objdevice->setdebug(true,true);  //force manual debug
  $snmpok=0;
  $result = array();
  $msg = "Error: Serial number no response";
  $Ciscotype= CiscoisCat29xx($objdevice);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Serial number Load MIB ->".print_r($a_xmlmib,true)."\n");
  if ($Ciscotype=="29xx") {
    $a_xmlmib = array(array("source"=>"CISCO-C2900-MIB.xml"));
    $mib="c2900PortDuplexStatus";
    $mib2="c2900PortSwitchPortIndex";
  } else {
   $a_xmlmib = array(array("source"=>"ciscoStackMIB.xml"));
   $mib="portDuplex";
   $mib2="portIfIndex";
  }
  searchin_xmlMIB($a_xmlmib,$mib,$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco module description MIB search of $mib ->".print_r($valmib,true)."\n");
  searchin_xmlMIB($a_xmlmib,$mib2,$valmib2);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco module Serialnumber MIB search of $mib2 ->".print_r($valmib2,true)."\n");
  $snmpok=(sizeof($valmib)>0) && (sizeof($valmib2)>0);
  if ($snmpok) {
   networkclass_give_snmp($objdevice,$valmib);
   $pDuplex = $objdevice->result;
   $snmpok1 = $objdevice->retcode;
   networkclass_give_snmp($objdevice,$valmib2);
   $pDuplexindex = $objdevice->result;
   $snmpok2 = $objdevice->retcode;
   $snmpok = $snmpok1 && $snmpok2;
   if ($snmpok) {
     if ((sizeof($pDuplex)>0) && (sizeof($pDuplexindex)>0)) {
        remove_oid_for_find_index($valmib["oid"],$pDuplex,$result1);
        remove_oid_for_find_index($valmib2["oid"],$pDuplexindex,$result2);
        foreach ($result2 as $index => $val) {
        	$result[$val] = $result1[$index];
        }
     }
     if ($Ciscotype=="29xx") {
     } else {
     }
    $msg = "";
   }
  }

 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG Cisco Serialnumber result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}
*/

function CiscoTFTPIOStransfert($objdevice) {
// IOS 12.0 and later
// reference http://www.cisco.com/en/US/tech/tk648/tk362/technologies_tech_note09186a0080094aa6.shtml
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $oidIOS = array(
                "method"          => '.1.3.6.1.4.1.9.9.96.1.1.1.1.2',
                "source"          => '.1.3.6.1.4.1.9.9.96.1.1.1.1.3',
                "destination"     => '.1.3.6.1.4.1.9.9.96.1.1.1.1.4',
                "ipaddress"       => '.1.3.6.1.4.1.9.9.96.1.1.1.1.5',
                "filename"        => '.1.3.6.1.4.1.9.9.96.1.1.1.1.6',
                "rowstatus"       => '.1.3.6.1.4.1.9.9.96.1.1.1.1.14',
                "state"           => '.1.3.6.1.4.1.9.9.96.1.1.1.1.10',
                "cause"           => '.1.3.6.1.4.1.9.9.96.1.1.1.1.13'
             );
  $filelocation = array( "tftp"    => '1',
                   "start"         => '3',
                   "startup"       => '3',
                   "run"           => '4'
                );

  $state = array( "waiting" => '1',
             "running" => '2',
             "success" => '3',
             "failed"  => '4'
           );
           
  $protocol = array(
                "tftp"  => '1',
                "ftp"  => '2',
                "rcp"  => '3',
                "scp"  => '4',
                "sftp"  => '5'
	   );
  $cause = array( "1" => 'Unknown Copy Failure',
             "2" => 'Bad File Name',
             "3" => 'Network timeout',
             "4" => 'Not Enough Memory',
             "5" => 'Source Configuration doesnt exist.'
           );
  $error ="";
  // generate random number used for mib instances
  $randnbr = rand(10,900);
  $thesource0= ($objdevice->tftpparam["source"]=='tftp') ? "tftp": $objdevice->tftpparam["filesrc"];
  $thesource = $filelocation[$thesource0];
  $thedest0= ($objdevice->tftpparam["source"]=='tftp') ? $objdevice->tftpparam["filedest"]:"tftp";
  $thedest = $filelocation[$thedest0];
  $thefile = ($objdevice->tftpparam["source"]=='tftp') ? $objdevice->tftpparam["filesrc"]:$objdevice->tftpparam["filedest"];
  if (($networkclass_isdebug)|| ($networkclass_islog))
     networkclass_debug("DEBUG CiscoTFTPIOStransfert: random number=$randnbr, "
         ."source=$thesource0, source's file=$thesource, destination=$thedest0, thedestination's file=$thedest"
         ."\n");
  // set up the request
      print "DEBUG CiscoTFTPIOStransfert: random number=$randnbr, "
         ."source=$thesource0, source's file=$thesource, destination=$thedest0, thedestination's file=$thedest"
         ."\n";
  $to_do = array (
 		// select method of transfer
 		array("oid" => $oidIOS["method"].".$randnbr", "type" => 'i', "value" => $protocol["tftp"])
 		// select source file location
 		,array("oid" => $oidIOS["source"].".$randnbr", "type" => 'i', "value" => $thesource)
 		// select destination file location
 		,array("oid" => $oidIOS["destination"].".$randnbr", "type" => 'i', "value" => $thedest)
 		// set tftpserver ip address
 		,array("oid" => $oidIOS["ipaddress"].".$randnbr", "type" => 'a', "value" => "".$objdevice->tftpparam["tftpip"])
 		// set the filename being written
 		,array("oid" => $oidIOS["filename"].".$randnbr", "type" => 's', "value" => $thefile)
 		// set the session status
 		,array("oid" => $oidIOS["rowstatus"].".$randnbr", "type" => 'i', "value" => '1')
		);
  $snmpok = $objdevice->set_list($to_do);
  $snmpok= $objdevice->retcode;
  if ($snmpok) {
		//set default status as "running"
		$result = $state["running"];
		$snmpok= true;
		// check for the results status
		$j=0;
		while ($snmpok && (($result == $state["running"]) || $result == $state["waiting"]) && ($j <10)) {
		  // get the current status of the tftp server's action
		  $objdevice->get($oidIOS["state"].".$randnbr");
		  $snmpok= $objdevice->retcode;
		  $ret_val = $objdevice->result;
		  if ($snmpok)
		     $result = $ret_val;
		   else $result= "";
		  sleep(1);
		  $j++; // not wait more than 10 seconds 
		}
  }

  // failure!
  if (!isset($result)) {
     	$ret_val= $error = "Error: SNMP Session failed during transfer";
	$snmpok = 0;
  } elseif (($result != '3')&&($result!="")) {
      	  $cause_req = $objdevice->get($oidIOS["cause"].".$randnbr");
     	  $ret_val= $error = $cause[$cause_req];
	  $snmpok = 0;
  } else {
		$ret_val= "success";
		$snmpok=1;
  }
  // clear the rowstatus for the remote device
  $clear = $objdevice->set($oidIOS["rowstatus"].".$randnbr","i","6");
  if (($networkclass_isdebug)|| ($networkclass_islog))
     networkclass_debug("DEBUG CiscoTFTPIOStransfert result: "
         ."result=$ret_val".(($snmpok)?"":$error)."\n");
  $objdevice->result  = $ret_val;
  $objdevice->errormsg  = $error;
  $objdevice->retcode = $snmpok;
}

function CiscoTFTPCatOStransfert($objdevice) {
// reference http://www.cisco.com/en/US/tech/tk648/tk362/technologies_tech_note09186a0080094aa3.shtml
//force debug $objdevice->setdebug(true,true);
 global $networkclass_isdebug,$networkclass_islog;
  $oidCatOS = array ("cat_ipaddress"   => '.1.3.6.1.4.1.9.5.1.5.1.0',
                "cat_filename"    => '.1.3.6.1.4.1.9.5.1.5.2.0',
                "cat_module"      => '.1.3.6.1.4.1.9.5.1.5.3.0',
                "cat_action"      => '.1.3.6.1.4.1.9.5.1.5.4.0',
                "cat_result"      => '.1.3.6.1.4.1.9.5.1.5.5.0',
                "cat_mod2stdbystatus" => '.1.3.6.1.4.1.9.5.1.3.1.1.21.2',
                "cat_modstatus"   => '.1.3.6.1.4.1.9.5.1.3.1.1.21'
            );
  $status = array ("other" => 1,
		   "active"=> 2,
		   "standby"=> 3,
		   "error" =>4
		   ); 
  $tftpResult = array ( 1     => 'inProgress',
                        2     => 'success',
                        3     => 'No Response',
                        4     => 'Too Many Retries',
                        5     => 'No Buffers',
                        6     => 'No Processes',
                        7     => 'Bad Checksum',
                        8     => 'Bad Length',
                        9     => 'Bad Flash',
                        10    => 'Server Error',
                        11    => 'User Canceled',
                        12    => 'Wrong Code',
                        13    => 'File Not Found',
                        14    => 'Invalid Tftp Host',
                        15    => 'Invalid Tftp Module',
                        16    => 'Access Violation',
                        17    => 'Unknown Status : Check TFTP Server',
                        18    => 'Invalid Storage Device',
                        19    => 'Insufficient Space On Storage Device',
                        20    => 'Insufficient Dram Size',
                        21    => 'Incompatible Image',
                   );
  $tftpactionCatOS = array ("other"   => '1',
                "downloadConfig"    => '2',
                "uploadConfig"      => '3',
                "downloadSw"      => '4',
                "uploadSw"      => '5',
                "downloadFw" => '6',
                "uploadFw"   => '7'
            );
  if ($objdevice->tftpparam["startup"]) {
  	$error = "Error: startup-config not possible using the CatOS";
     	$ret_val= $error;
	$snmpok = 0;
  } else {
    $tftpmod = "1";
    // search the active supervisor module
    $curoid = $oidCatOS["cat_modstatus"];
    // go through each module's status until we find the active supervisor
    $snmpok = false;
    for ($l = 0; $l < 16; $l++) {
	   //get the next module oid in the stack
	   $objdevice->getnext($curoid);
	   $ret_val = $objdevice->result;
	   $snmpok = $objdevice->retcode;
          if ($snmpok) {
		// are we in the correct block of oids?
		$results = $ret_val;
		$search = '/^'.$oidCatOS["cat_modstatus"].'/';
		if (0<preg_match($search, $results[0])) {
		  // is this the actice sup?
		  $curoid = $results[0];
		  // Value of 2 means this is the active sup
		  if ($ret_val[$curoid] == $status["active"]) {
		    // sup # is the last element of the oid
		    $oidparts = split(".", $results[0]);
		    $tftpmod = "".$oidparts[count($oidparts)];
		    break;
		  }
        	} else { break; }
	  } else {
		// try the defaults if SNMP fails 
		break;
	  }
    }
    // set up the request
    $source = ($objdevice->tftpparam["source"]=='tftp') ? $tftpactionCatOS["downloadConfig"] : $tftpactionCatOS["uploadConfig"];
    $thefile = ($objdevice->tftpparam["source"]=='tftp') ? $objdevice->tftpparam["filesrc"]:$objdevice->tftpparam["filedest"];
    $to_do = array (
 	// IP address of the TFTP server
 	array("oid" => $oidCatOS["cat_ipaddress"], "type" => 's', "value" => $objdevice->tftpparam["tftpip"])
 	// name of the switch configuration file
 	,array("oid" => $oidCatOS["cat_filename"], "type" => 's', "value" => $thefile)
 	// Module number. In this case, Supervisor module
 	,array("oid" => $oidCatOS["cat_module"], "type" => 'i', "value" => $tftpmod)
 	// TFTP action.
 	,array("oid" => $oidCatOS["cat_action"], "type" => 'i', "value" => $source)
      );
    $objdevice->set_list($to_do);
    $snmpok2 = $objdevice->retcode;
    $snmpok2= false;
    if ($snmpok) {
		//set default status as "running"
		$result = "1";
		$snmpok2= true;
		// check for the results status
		while ($snmpok2 && ($result == "1")) {
		  // get the current status of the tftp server's action
		  $objdevice->get($oidCatOS["cat_result"]);
		  $ret_val = $objdevice->result;
		  $snmpok2 = $objdevice->retcode;
		  $error   = $objdevice->errormsg;
		  if ($snmpok2) $result = $ret_val; else $result= "";
		}
   }
   // failure!
   if ($snmpok2) {
     	$ret_val= $error ="Error: SNMP Session failed during transfer";
	$snmpok = 0;
   } elseif (($result != '2')&&($result!="")) {
     	  $ret_val= $error =$tftpResult[$result];
	  $snmpok = 0;
   } else {
	$ret_val= $error= $tftpResult["2"];
	$snmpok=1;
   }
  }
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG CiscoTFTPCatOStransfert: ".(($snmpok)?"OK":"BAD")."\n");
  $objdevice->result  = $ret_val;
  $objdevice->errormsg  = $error;
  $objdevice->retcode = $snmpok;
}

function CiscoTFTPtransfer_deprecated($objdevice)
{
// http://www.cisco.com/en/US/tech/tk648/tk362/technologies_tech_note09186a008009463e.shtml
//force debug $objdevice->setdebug(true,true);
 global $networkclass_isdebug,$networkclass_islog;
  $oidIOSdepreciated = array ( //deprecated system mibs
                 "wrnet"    => '.1.3.6.1.4.1.9.2.1.55.'
                ,"confnet"  => '.1.3.6.1.4.1.9.2.1.53.'
            );
  if ($objdevice->tftpparam["startup"]) {
    $ret_val= $error = "Error: Copying configurations to and from startup-config\nis not possible using deprecated mibs.";
    $snmpok = 0;
  } else {
    $mib = ($objdevice->tftpparam["source"]=='tftp') ? $oidIOSdepreciated["confnet"]: $oidIOSdepreciated["wrnet"];
    $mib = $mib.$objdevice->tftpparam["tftpip"]; 
    $thefile = ($objdevice->tftpparam["source"]=='tftp') ? $objdevice->tftpparam["filesrc"]:$objdevice->tftpparam["filedest"];
    $to_do = array (
 		    // IP address of the TFTP server & file
 		     array("oid" => $mib, "type" => 's', "value" => $thefile)
	);
    // these devices are very slow 
    $oldretries=$objdevice->param["retries"];
    $objdevice->param["retries"] = 0;   // because the hardware is old and slow
    $oldtimeout=$objdevice->param["timeout"];
    $objdevice->param["timeout"] = 2*$objdevice->param["timeout"];   // because the hardware is old and slow
    $snmpok = $objdevice->set_list($to_do);
    $objdevice->param["retries"] = $oldretries;
    $objdevice->param["timeout"] = $oldtimeout;
    if (($networkclass_isdebug)|| ($networkclass_islog))
     networkclass_debug("DEBUG CiscoTFTPtransfer_deprecated: ".(($snmpok)?"OK":"BAD")."\n");
    if (!$snmpok)
     	$ret_val= $error = "Error: SNMP Session failed during transfer.";
    else $ret_val= "";
  }
  $objdevice->result  = $ret_val;
  $objdevice->retcode = $snmpok;
  $objdevice->errormsg  = $error;
}

/* ************************************************************************
 TrapEvent : should return if the router has been changed in analysing the message of the syslog
 $result["changed"] = "running" "startup" or ""
 $result["host"] = host that has been changed;
 $result["date"] = 
 .....
!!!!!!!!!!!!! UNDER CONSTRUCTION !!!!!!!!!!!!!!!!!!!!!!!
*/
function CiscoTrapEvent($objdevice,$syslogmsg="") {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
/*
ccmHistoryEventConfigSource 
OID 1.3.6.1.4.1.9.9.43.1.1.6.1.4 
ccmHistoryEventConfigDestination 
OID 1.3.6.1.4.1.9.9.43.1.1.6.1.5 
1:erase
2:commandSource
3:running
4:startup
5:local
6:networkTftp
7:networkRcp

ccmHistoryEventCommandSource 
OID 1.3.6.1.4.1.9.9.43.1.1.6.1.3 
Type INTEGER 
Permission read-only 
Status current 
Values 1 : commandLine
2 : snmp
*/
 $mib="cpmCPUTotalTable";
 $a_xmlmib = array(array("source"=>"CISCO-CONFIG-MAN-MIB.xml"));
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG CiscoTrapEvent MIB ->".print_r($a_xmlmib,true)."\n");
 $snmpok = 1; // default always OK but no trap
 $result = array("changed" => ""
                ,"host" => ""
                ,"date" => ""
                );
 $msg    = "Error: no trap configuration events";
 searchin_xmlMIB($a_xmlmib,$mib,$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG CiscoTrapEvent MIB search of $mib ->".print_r($valmib,true)."\n");
 $snmpok=(sizeof($valmib)>0);
 if ($snmpok) {
 }
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

?>