<?php
/*
 +-------------------------------------------------------------------------+
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/
/*******************************************************************************
    Author ......... Jean-Michel Pepin classes plugin for network devices (PEPJ in forum.cacti.net & http://cactiusers.org/forums/)
    Modified ....... .
    Program ........ class definition for HP network devices in PHP for Cacti or PHP software

    Thanks to all the developpers of cacti who make my work much easier.
*******************************************************************************/
/* define the OID of the vendor */
$toidvendor['hp']           = '.1.3.6.1.4.1.11';
$toidtovendor['.1.3.6.1.4.1.11'] = 'hp';

class hpnetworkclass
{
   function vendorname($FunctionName)
   {
        return "hp";       
   }
   function showclass()
   {
        return "this is a hp class";       
   }
   function priv($obj,$FunctionName,$param="")
   {
   	$fct = "hp".$FunctionName;
   	if (function_exists($fct)) $ret = ($param=="") ? $fct($obj) : $fct($obj,$param);
       	else {
       	  $ret=0;
          $obj->result= array();
          $obj->errormsg = "Error: Function does not exist.";
          $obj->retcode = $ret;
       	}
        return $ret;
   }
   function tftp($obj)
   {
	hpTFTPtransfer($obj);
   }
   /* return if the device is a host */
   function DeviceIsHost($objdevice,$init=false) {
     global $networkclass_isdebug,$networkclass_islog;
     // example for HP printer

     // the two following value could help you to find it
     // $obj->description();  // to be sure description has been ask at least one time
     // $description = $obj->result; or $description = $obj->description;
     // $oiddevice = $objdevice->oiddevice;
     //force debug $objdevice->setdebug(true,true);
     // if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG DeviceIsHost ->".print_r($valmib,true)."\n");
     $oiddevice = $objdevice->oiddevice;
     //$objdevice->description();  // to be sure description has been ask at least one time
     //$description = $objdevice->result;
     if ($init) {
        $objdevice->ishost = (substr_count($oiddevice,"1.3.6.1.4.1.11.2.3.7")<1);
     }
     $result= $objdevice->ishost;
     return $result;
   }
}

/* return the type of the device (used only for the function in this class in case of a specific behaviour)
*/
function hpChecktypdevice($obj) {
   $description  = $obj->description;
   $oiddevice  = $obj->oiddevice;
   $typdevice = "unknow";
   switch (true) {
	case (substr_count($oiddevice,"1.3.6.1.4.1.11.2.3.7.1")>0):
	  $typdevice = "bridge";
	  break;
	case (substr_count($oiddevice,"1.3.6.1.4.1.11.2.3.7.2")>0):
	  $typdevice = "router";
	  break;
	case (substr_count($oiddevice,"1.3.6.1.4.1.11.2.3.7.5")>0):
	  $typdevice = "hub";
	  break;
	case (substr_count($oiddevice,"1.3.6.1.4.1.11.2.3.7.8")>0):
	  $typdevice = "chassis";
	  break;
	case (substr_count($oiddevice,"1.3.6.1.4.1.11.2.3.7.11")>0):
	  $typdevice = "switch";
	  break;
	default:
	  $typdevice = "other";
	  break;
   }
   return $typdevice;
}

/* ************************************************************************
 Duplex : should return an array like
 $result[ifindex] = 0 for unknow, 1 for half duplex, 2 for full duplex, 3 auto;
*/
function hpDuplex($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $snmpok=0;
 $result = array();
 $msg = "Error: Duplex no response";
 // only for switches
 if (hpChecktypdevice($objdevice)=="switch") {
  $mib="ifSpecific";
  $a_xmlmib = array(array("source"=>"RFC1213-MIB.xml"));
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp Duplex Load MIB ->".print_r($a_xmlmib,true)."\n");
  searchin_xmlMIB($a_xmlmib,$mib,$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp Duplex MIB search of $mib ->".print_r($valmib,true)."\n");
  $snmpok=(sizeof($valmib)>0);
  if ($snmpok) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok = $objdevice->retcode;
   if ($snmpok) {
     if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$result);
     // conversion for independancy reason
     foreach ($result as $index => $val) {
      switch ($val) {
	case 11: // 11->10BASE-T  full duplex 16->100BASE-TX full duplex 20->100BASE-T2 full duplex
	case 20:
	  $res= 2;
	  break;
	case 10: // 10->10BASE-T  half duplex 14->100BASE-T4, 15->100BASE-TX half duplex 19->100BASE-T2 half duplex
	case 14:
	case 15:
	case 19:
	  $res= 1;
	  break;
	default:    //  0       other or unknown
	   $res= 0;
	  break;
      }
      $result[$index] =$res;
     }
     $msg = "";
   }
  }
 } else {
 	// printer , server, ...
 }
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp Duplex result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 Trunk : should return an array like in order to be independant for vendors
 $result[interfaceindex] = "0" no trunk "1","2",... for other state;
*/
function hpTrunk($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);

 $snmpok=0;
 $result= array();
 $msg = "Error: no response";
 // only for switches
 if (hpChecktypdevice($objdevice)=="switch") {
  $a_xmlmib = array(array("source"=>"hp-mib.xml"));
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp Trunk Load MIB ->".print_r($a_xmlmib,true)."\n");
  $mib="hpVlanMemberTagged2";
  searchin_xmlMIB($a_xmlmib,$mib,$valmib);
  if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok=$objdevice->retcode;
   if ($snmpok) {
       $result= $objdevice->result;
       /* HP give tagged 1 nottagged 2 no 3 auto 4 */
       /* we want "0" no trunk "1","2",... for other state" */
       foreach ($result as $index => $value) if ($result[$index]=="3") $result[$index]="0";
       $msg = "";
     } else {
    	// no error but no trunk no vlan
    	$msg = "Error: MIB hp Trunk not found.";
    	$snmpok= 1;
     }
   }
  } else $msg = "Error: no MIB found";
 $objdevice->result= $result;
 if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG hp Trunk result ->".print_r($result,true)."\n");
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 VLAN : should return an array like in order to be independant for vendors
 $result["state"][vlanindex] = "0" for no VLAN or VLAN number;
 $result["VlanID"][vlanindex] = "0" for no VLAN or VLAN number;
 $result["index"][vlanindex] = ifindex;
 $result["nbport_by_vlan"][vlanindex] = nb ports in vlan
 $result["port_by_vlan"][vlanindex][] = array of ports in vlan
 $result["vlan_by_port"][vlanindex][] = array of ports in vlan
 $result["otherinfos"][] = array of other infos
 */
function hpVLAN($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $vlanstate= array(); 
 $vlanname = array(); 
 $vlanindex = array();
 $arrayofnbportsinvlan = array();
 $arrayofportsinvlan = array();
 $snmpok=0;
 $msg = "Error: MIB not exist.";
 $a_xmlmib = array(array("source"=>"hp-mib.xml"));
 /* search the VLAN information */
 searchin_xmlMIB($a_xmlmib,"hpVlanIdentStatus",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG HP VLAN state MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok1=$objdevice->retcode;
   /* give activated(1) notActivated(2 or 3) */
   /* we want "0" no vlan active "1","2",... for other state" */
   if ($snmpok1) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$vlanstate);
        foreach ($vlanstate as $index => $value) if ($vlanstate[$index]!="1") $vlanstate[$index]="0";
        $msg= "";
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG HP VLAN state array ->".print_r($vlanstate,true)."\n");
 }
 searchin_xmlMIB($a_xmlmib,"hpVlanIdentName",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG HP VLAN name MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok2=$objdevice->retcode;
   if ($snmpok2) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$vlanname);
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG HP VLAN vlanname ->".print_r($vlanname,true)."\n");
 }
 searchin_xmlMIB($a_xmlmib,"hpVlanDot1QID",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG HP VLAN VLanId MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok3=$objdevice->retcode;
   if ($snmpok3) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$vlanindex);
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG HP VLAN VLanId ->".print_r($vlanindex,true)."\n");
 }

 searchin_xmlMIB($a_xmlmib,"hpVlanMemberIndex",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG HP Port hpVlanMemberIndex MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok3=$objdevice->retcode;
   if ($snmpok3) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$arrayofvlanlist);
   	foreach ($vlanindex as $i => $vlan) {
   	 $nbr=0;
   	 $arrayofportsinvlan[$vlan]=array(); 
   	 foreach ($arrayofvlanlist as $ifndx => $ifvlan) {
   	  if ($ifvlan==$vlan) {
   	  	$nbr=$nbr+1;
   	  	$arrayofportsinvlan[$vlan][$nbr]= $ifndx;
   	  }
   	 }
   	 $arrayofnbportsinvlan[$vlan]=$nbr;
   	}
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG HP VLAN Port hpVlanMemberIndex ->".print_r($vlanindex,true)."\n");
 }
 /* convert to ifindex !!!!!*/
 $result["index"] = @$vlanindex;
 $result["state"] = @$vlanstate;
 $result["VlanID"] = @$vlanname;
 $result["nbport_by_vlan"] = @$arrayofnbportsinvlan;
 $result["port_by_vlan"] = @$arrayofportsinvlan;
 $result["vlan_by_port"] = array();
 foreach ($arrayofportsinvlan as $vlanID => $tport) {
    foreach ($tport as $ndx => $port) {
     $result["vlan_by_port"][$port][] = $vlanID;
    }
 }
 $objdevice->result= $result;
// if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG HP VLAN result ->".print_r($result,true)."\n");
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok1;
 return $snmpok1;
}

/* ************************************************************************
 Channel : should return an array with all type of channel like ... in order
  to be independant for vendors who have severals channel types
 $result["channelindex"][0..x]=channeltype (at least "LACP")
 $result["LACP"]["AdminKey"][ifIndex] = AdminKey of LaCP channel;  // "dot3adAggPortActorAdminKey"
 $result["LACP"]["OperKey"][ifIndex] = OperKey of LaCP channel;    // "dot3adAggPortActorOperKey"
 $result["LACP"]["PortActorAdminState"][ifIndex] = Port Selected in which index of the LaCP channel // "dot3adAggPortActorAdminState"
 $result["LACP"]["PortActorOperState"][ifIndex] = Port Attached in which index of the LaCP channel // "dot3adAggPortActorOperState"
 
 $result["other type of channel"]["PortActorOperState"][ifIndex] = ...;     "..."
 $result["other type of channel"]["pagpGroupIfIndex"][ifIndex] = Port Attached in which index of the ... channel or 0 for not attached
 ....
 */
function hpChannel($objdevice) {
 networkclass_LACP($objdevice);
 $snmpok1 = $objdevice->retcode;
 $lacp = $objdevice->result;
 $result["channelindex"][0] = "LACP";
 $result["LACP"] = $lacp;
 $snmpok = $snmpok1;
 $objdevice->result =$result;
 if ($snmpok) $objdevice->errormsg = "";
 return $snmpok;
}

/* ************************************************************************
 LACP : should return an array like in order to be independant for vendors
 per default IEEE8023-LAG-MIB
 $result["AdminKey"][ifIndex] = AdminKey of LaCP channel;   "dot3adAggPortActorAdminKey"
 $result["OperKey"][ifIndex] = OperKey of LaCP channel;     "dot3adAggPortActorOperKey"
 $result["PortActorAdminState"][ifIndex] = Port Selected in which index of the LaCP channel
 $result["PortActorOperState"][ifIndex] = Port Attached in which index of the LaCP channel
 */
function hpLACP($objdevice) {
 networkclass_LACP($objdevice);
 return $objdevice->retcode;
}

/* CPU : should return an array like
 $result["firstaveragename"][0] = value 1. average;
 $result["2averagename"][0] = value 2. average;
 $result["3averagename"][0] = value 3. average;
*/
function hpCPU($objdevice) {
 // ??? cpqHeTemperatureTable 1.3.6.1.4.1.232.6.2.6.8
 $result["1Sec"] = 0;
 $result["5Sec"] = 0;
 $result["1min"] = 0;
 $ret=0;
 $msg = "Error: CPU status not enabled or not supported.";
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $ret;
 return $ret;
}

/* ************************************************************************
 Temperature : should return an array of temperature (in degres Celsius) like
 $result["modulename"][0] = modulename; or ""
 $result["mean"][0] = 0;
 $result["min"][0] = value; or ""
 $result["max"][0] = value; or ""
 $result["state"][0] = value; or ""
PS: Fahrenheit to celsius (F - 32) / 1.8
*/
function hpTemperature($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//$objdevice->setdebug(true,true);
 $a_xmlmib = array(array("source"=>"HP-SN-AGENT-MIB.xml"));
 $snmpok=0;
 $result=array();
 $msg = "Error: hp Temperature Status Table not enabled or not supported.";
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp temperature Load MIB ->".print_r($a_xmlmib,true)."\n");
 searchin_xmlMIB($a_xmlmib,"snChasActualTemperature",$valmib);
 if (($networkclass_isdebug) || ($networkclass_islog) ) networkclass_debug("DEBUG hp temperature MIB search ->".print_r($valmib,true)."\n");
 $snmpok=(sizeof($valmib)>0);
 if ($snmpok) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok = $objdevice->retcode;
   if ($snmpok) {
     $tempcur = intval($objdevice->result/2); // because unit of 0.5 degres
     if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp temperature current ->".$tempcur.", is ok?".intval($snmpok)."\n");
     $result["mean"][0] = $tempcur;
     $msg = "";
   }
 }
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 Fan : should return an array of Fan state () like
 $result["description"][index] = decription where index is the index number of the fan
 $result["state"][index] = "ok" or "down" "not present" are mandatory for functioning / not functioning , or an another text
*/
function hpFan($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $a_xmlmib = array(array("source"=>"HP-SN-AGENT-MIB.xml"));
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp Fan Load MIB ->".print_r($a_xmlmib,true)."\n");
 $snmpok=0;
 $msg = "hp snChasFanDescription not enabled or not supported.";
 $mib = "snChasFanDescription";
 searchin_xmlMIB($a_xmlmib,$mib,$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp Fan description MIB search of $mib ->".print_r($valmib,true)."\n");
 $snmpok1=(sizeof($valmib)>0);
 networkclass_give_snmp($objdevice,$valmib);
 if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$tdescription);
 else  $tdescription= array();
 $ok1=$objdevice->retcode;
 $tstate = array();
 if ($ok1) {
   $mib="snChasFanOperStatus";
   searchin_xmlMIB($a_xmlmib,$mib,$valmib);
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp Fan state MIB search of $mib ->".print_r($valmib,true)."\n");
   networkclass_give_snmp($objdevice,$valmib);
   if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$tstate);
   $ok2=$objdevice->retcode;
   if ($ok2) {
     foreach($tstate as $index => $val) {
      switch ($val) {
        // by other1 normal 2 failure 3
      	case 1:
      	  $tstate[$index] = "not present";
      	  break;
      	case 2:
      	  $tstate[$index] = "ok";
      	  break;
      	case 3:
      	  $tstate[$index] = "critical";
      	  break;
      	default:
      	  $tstate[$index] = "not Functioning";
      }
     }
   }
 }
 $snmpok = $ok1 && $ok2;
 $result["description"] = $tdescription;
 $result["state"] = $tstate;
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG hp Fan result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 RAM : should return an array of RAM 
 $result["RAM"] = 
 $result["RAMused"] = 
*/
function hpRAM($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
  $result = array();
  $ret=0;
  $msg = "Error: RAM status table not enabled or not supported.";
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG HP RAM not enabled or not supported"."\n");
  $objdevice->result= $result;
  $objdevice->errormsg = $msg;
  $objdevice->retcode = $ret;
  return $ret;
}

/* ************************************************************************
 MAC : for the VlanID should return an array of MAC addresses like
 $result[interfaceindex][indexMAC] -> Macaddress where interfaceindex is the inteface index
*/
function hpMAC($objdevice,$vlanID) {
 networkclass_MAC($objdevice,$vlanID);
 return $objdevice->retcode;
}

/* ************************************************************************
 STPchanges : for the VlanID should return an array of 
 $result["TopChanges"] -> The total number of topology changes detected by this bridge since the management entity was last reset or initialized.
 $result["TimeSinceTopologyChange"] -> The time (in hundredths of a second) since the last time a topology change was detected by the bridge entity.
*/
function hpSTPchanges($objdevice,$vlanID="") {
 networkclass_STPchanges($objdevice,$vlanID);
 return $objdevice->retcode;
}

function hpTFTPtransfer($objdevice) {
   $ret_val= "Error: SNMP Session failed during transfer. function in developpement .......";
   $ret_val= $msg= "Error: function in developpement .......";
   $snmpok = 0;
   $objdevice->result  = $ret_val;
   $objdevice->errormsg = $msg;
   $objdevice->retcode = $snmpok;
}

?>